<?php

namespace Waggybytes\SimplyRemindMe\Library\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Redis;

class MediaType extends Model
{
    public const string EMAIL_CODE = 'email';
    public const string PUSH_CODE = 'push';
    public const string SMS_CODE = 'sms';
    public const string VOICE_CODE = 'voice';

    protected $fillable = ['code_name', 'name', 'class_handler', 'queue_order'];

    protected $hidden = ['class_handler', 'queue_order', 'created_at', 'updated_at'];

    public static function email()
    {
        return self::getByCodeUsingCache(self::EMAIL_CODE);
    }

    public static function pushNotification()
    {
        return self::getByCodeUsingCache(self::PUSH_CODE);
    }

    public static function sms()
    {
        return self::getByCodeUsingCache(self::SMS_CODE);
    }

    public static function voice()
    {
        return self::getByCodeUsingCache(self::VOICE_CODE);
    }

    private static function getByCodeUsingCache($code)
    {
        $cacheKey = sprintf('mediaType.%s', $code);

        $result = Redis::get($cacheKey);
        if (!is_null($result))
        {
            return unserialize($result);
        }

        $mediaType = MediaType::where('code_name', $code)->first();
        if (is_null($mediaType))
        {
            throw new \Exception(sprintf('Media type with code \'%s\' could not be found', $code));
        }

        Redis::set($cacheKey, serialize($mediaType));
        Redis::expire($cacheKey, 86400);
    }
}